from distutils.core import setup

import os
from distutils.core import setup
from itertools import chain

__all__ = ['setup_recursive']

def get_package_dir(pkg, attrs):
    path = pkg.split('.')
    package_dir = attrs['package_dir']

    try:
        return package_dir[pkg]
    except KeyError:
        pass

    for i in range(1, len(path)):
        try:
            pdir = package_dir['.'.join(path[:-i])]
        except KeyError:
            pass
        else:
            return os.path.join(*chain([pdir], path[-i:]))
    else:
        rootpdir = package_dir.get('')
        if rootpdir is not None:
            path.insert(0, rootpdir)
        return os.path.join(*path)

def packages_recursive(basepkg, attrs):
    pkgdir = get_package_dir(basepkg, attrs)
    if pkgdir == '':
        pkgdir = '.'

    for root, dirs, files in os.walk(pkgdir):
        if '__init__.py' in files:
            # note, relroot has a leading / here if non-empty
            relroot = root[len(pkgdir):]
            yield basepkg + relroot.replace(os.path.sep, '.')

            # skip directories with a . (which includes hidden files) and
            # PEP 3147 cache dirs
            dirs[:] = [d for d in dirs if not '.' in d and d != '__pycache__']
        else:
            # don't recurse into non-package dirs
            dirs[:] = []

def files_recursive(pkg, entries, attrs):
    def make_native(path):
        return path.replace('/', os.path.sep)
    def make_slash(path):
        return path.replace(os.path.sep, '/')

    pkgdir = get_package_dir(pkg, attrs)

    for e in entries:
        fe = os.path.join(pkgdir, make_native(e))
        if os.path.isfile(fe):
            yield make_slash(fe)
        elif os.path.isdir(fe):
            for root, dirs, files in os.walk(fe):
                if pkgdir:
                    relroot = root[len(pkgdir) + 1:]
                else:
                    relroot = root
                for f in files:
                    yield make_slash(os.path.join(relroot, f))

                # skip hidden and PEP 3147 cache dirs
                dirs[:] = [d for d in dirs if not d.startswith('.') and d != '__pycache__']

def setup_recursive(**attrs):
    attrs.setdefault('package_dir', dict())
    attrs.setdefault('package_data', dict())
    attrs.setdefault('packages', list())

    # distutils understands '' as '.' in package_dir. To make this convention
    # work for setup_recursive, too, just substitute all '' entries by '.' at
    # the beginning.
    attrs['package_dir'] = dict((pkg, path if path != '' else '.') for (pkg, path) in attrs['package_dir'].items())

    if 'rec_packages' in attrs:
        attrs['packages'].extend(list(chain(*(packages_recursive(pkg, attrs) for pkg in attrs['rec_packages']))))
        del attrs['rec_packages']
    if 'rec_package_data' in attrs:
        attrs['package_data'].update(dict((pkg, list(files_recursive(pkg, entries, attrs))) for (pkg, entries) in attrs['rec_package_data'].items()))
        del attrs['rec_package_data']

    setup(**attrs)
